extern int force_linkat (int, char const *, int, char const *, int, bool, int)
  _GL_ATTRIBUTE_NONNULL ();
extern int force_symlinkat (char const *, int, char const *, bool, int)
  _GL_ATTRIBUTE_NONNULL ();
