#!/bin/sh
# Ensure that runcon -c uses absolute file names

# Copyright (C) 2022-2023 Free Software Foundation, Inc.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

. "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
print_ver_ runcon

# Create an executable that's sure to fail
printf '%s\n' '#!/bin/sh' 'exit 1' >> 'true' || framework_failure_
chmod a+x 'true' || framework_failure_

runcon -c true; ret=$?
test "$ret" = 125 && skip_ 'runcon setup failed'
test "$ret" = 1 || fail=1

Exit $fail
